//The AICarManager object is a script object used
//for updating AIWheeledVehicles. It allows them to
//follow in game paths without modification to the
//source code of the engine.

//By Quentin Headen

new ScriptObject(AICarManager)
{
   botGroup = 0; //A SimGroup object containing all of the ai cars (bots)
   arrivedDistance = 8; //The distance from the destination that the bot can say it arrived
   updateSpeed = 100; //The speed at which the manager rechecks the bots
   botMoveSpeed = 0.5; //The movement speed of the bots. Start at half.
   thinkSchedule = 0; //The schedule to update the thinking
};

function AICarManager::think(%this)
{   
   //If the bot group isn't set
   if(!isObject(%this.botGroup))
   {
      //Show an error and return
      error("You must set the botGroup SimGroup using setBotGroup before thinking can occur!");
      return;
   }
   
   //For each vehicle bot in the bot group
   for(%i = 0; %i < %this.botGroup.getCount(); %i++)
   {
      //Get the bot object
      %bot = %this.botGroup.getObject(%i);
      
      //If the path for this vehicle bot
      //is set
      if(isObject(%bot.path))
      {
         //Check the distance from the bot to its current path node
         %distance = VectorDist(%bot.getPosition(), %bot.path.getObject(%bot.pathNode).getPosition());
         
         //If the distance is less than the arrived distance
         if(%distance < %this.arrivedDistance)
         {
            //Move to the next node
            %bot.pathNode++;
            
            //If that node dosen't exist
            if(%bot.pathNode > %bot.path.getCount() - 1)
            {
               //Move back to the beginning node
               %bot.pathNode = 0;
            }
            
            //Set the updated move destination
            %bot.setMoveDestination(%bot.path.getObject(%bot.pathNode).getPosition());
         }
      }
   }
   
   //Continue to think
   %this.thinkSchedule = %this.schedule(%this.updateSpeed, think);
}

function AICarManager::startBot(%this, %bot, %path)
{
   //If the bot group doesn't exist
   if(!isObject(%this.botGroup))
   {
      //Create it
      %this.botGroup = new SimGroup();
   }
   
   //If this bot isn't a member of the bot group
   if(!%this.botGroup.isMember(%bot))
   {
      //Add it
      %this.botGroup.add(%bot);
   }
   
   //Set the starting path and path node for the bot
   %bot.path = %path;
   %bot.pathNode = 0;
   
   //Start the car to moving
   %bot.setMoveSpeed(%this.botMoveSpeed);
   %bot.setMoveDestination(%path.getObject(0).getPosition());
}

function AICarManager::stopBot(%this, %bot)
{
   //If the bot is in the bot group
   if(%this.botGroup.isMember(%bot))
   {
      //Stop it
      %bot.stop();
      
      //Remove it from the manager
      %this.botGroup.remove(%bot);
   }
}

function AiCarManager::stopThinking(%this)
{
   //Cancel the updating of the think() function
   cancel(%this.thinkSchedule);
   
   //For each car in the bot group
   for(%i = 0; %i < %this.botGroup.getCount(); %i++)
   {
      //Stop it
      %this.botGroup.getObject(%i).stop();
   }
}