//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// AIPlayer callbacks
// The AIPlayer class implements the following callbacks:
//
//    PlayerData::onStuck(%this,%obj)
//    PlayerData::onUnStuck(%this,%obj)
//    PlayerData::onStop(%this,%obj)
//    PlayerData::onMove(%this,%obj)
//    PlayerData::onReachDestination(%this,%obj)
//    PlayerData::onTargetEnterLOS(%this,%obj)
//    PlayerData::onTargetExitLOS(%this,%obj)
//    PlayerData::onAdd(%this,%obj)
//
// Since the AIPlayer doesn't implement it's own datablock, these callbacks
// all take place in the PlayerData namespace.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Demo Pathed AIPlayer.
//-----------------------------------------------------------------------------

datablock PlayerData(DemoPlayer : PlayerBody)
{
   shootingDelay = 2000;
};

datablock PlayerData(MrJohnson : PlayerBody)
{
   shapeFile = "~/data/shapes/CubixStudioMaleNPC/CubixStudioMaleNPC.dts";
   debrisShapeName = "~/data/shapes/CubixStudioMaleNPC/CubixStudioMaleNPC.dts";
};

datablock PlayerData(MrBlue : PlayerBody)
{
   shapeFile = "~/data/shapes/blueguy/player.dts";
   debrisShapeName = "~/data/shapes/blueguy/debris_player.dts";
};

datablock PlayerData(MrNew : PlayerBody)
{
   shapeFile = "~/data/shapes/newguy/newguy.dts";
};

function DemoPlayer::onReachDestination(%this,%obj)
{
   // Moves to the next node on the path.
   // Override for all player. Normally we'd override this for only
   // a specific player datablock or class of players. 
   if (%obj.path !$= "") {
      if (%obj.currentNode == %obj.targetNode)
         %this.onEndOfPath(%obj,%obj.path);
      else
         %obj.moveToNextNode();
   }
}

function DemoPlayer::onEndOfPath(%this,%obj,%path)
{
   %obj.nextTask();
}

function DemoPlayer::onEndSequence(%this,%obj,%slot)
{
   echo("Sequence Done!");
   %obj.stopThread(%slot);
   %obj.nextTask();
}


//-----------------------------------------------------------------------------
// AIPlayer static functions
//-----------------------------------------------------------------------------

function AIPlayer::spawn(%name,%spawnPoint,%style)
{
   // Create the demo player object
   if(%style == 0) {
      %player = new AiPlayer() { // GRUNT
         dataBlock = PlayerBody;
         botID = 0;
         style = %style; // Enemy Type
         accuracy = 2; // For random vector
         isAI = true; // Designated as AI
         isDead = false;
      };
   }else if(%style == 1) { // SNIPER
      %player = new AiPlayer() {
         dataBlock = PlayerBody;
         botID = 0;
         style = %style;
         accuracy = 1;
         isAI = true;
         isDead = false;
      };
   }else if(%style == 2) { // FRIENDLY
      %player = new AiPlayer() {
         dataBlock = MrNew;
         botID = 0;
         style = %style;
         accuracy = 1;
         isAI = true;
         isFriendly = true;
         isDead = false;
         AITBusy = false;
         AITBusyText = 'Shut your mouth fool';
         AITTalkingTo = 0;
         AITStartQuery = 1;
      };
   }
   MissionCleanup.add(%player);
   %player.setShapeName(%name);
   %player.setTransform(%spawnPoint);
   %player.botID = %player;
   if(%player.style != 2) {
      %player.mountImage(CrossbowImage,0);
      updateBot(%player.botID);
   }else{
      if(%name $= "Mr. Johnson") {
         %player.AITScript = "mrJohnson";
         %player.AITMugshot = "mrJohnson.png";
      }else if(%name $= "Whizzer") {
         %player.AITScript = "whizzer";
         %player.AITMugshot = "mrJohnson.png";
      }else if(%name $= "McGreary") {
         %player.AITScript = "mcgreary";
         %player.AITMugshot = "mrJohnson.png";
      }else if(%name $= "Chuck") {
         %player.AITScript = "chuck";
         %player.AITMugshot = "mrJohnson.png";
      }else if(%name $= "Timmy") {
         %player.AITScript = "timmy";
         %player.AITMugshot = "mrJohnson.png";
      }
   }
   return %player;
}

function updateBot(%id) {
   // Only run if-statement if client player is initialized   
   if($gspStartAI == true) { 
      %id.incInventory(CrossbowAmmo, 1);
      %botX = getWord(%id.getTransform(),0);
      %botY = getWord(%id.getTransform(),1);
      %botZ = getWord(%id.getTransform(),2);
      %playerX = getWord($gspPlayer.getTransform(),0);
      %playerY = getWord($gspPlayer.getTransform(),1);
      %playerZ = getWord($gspPlayer.getTransform(),2);
      %distance = mSqrt((%playerX - %botX)*(%playerX - %botX) + (%playerY - %botY)*(%playerY - %botY));
      switch(%id.style) {
         case 0: // GRUNT AI
            if(%distance <= 30) {
               %id.fire(true);
               //echo("Firing");
               %id.setMoveDestination(%id.getTransform());
            }else if(%distance > 80) {
               %id.fire(false);
               //echo("Stopped");
               if(%id.getDamageLevel() > 0)
               {
                  %id.setMoveDestination($gspPlayer.getTransform());
               }else{
                  %id.setMoveDestination(%id.getTransform());
               }
            }else{
               %id.fire(false);
               //echo("Moving");
               %id.setMoveDestination($gspPlayer.getTransform());
            }
            %id.aimAt($gspPlayer);
         case 1: // SNIPER AI
            if(%distance < 80) {
               %id.fire(true);
            }else{
               if(%id.getDamageLevel() > 0)
               {
                  %id.fire(true);
               }else{
                  %id.fire(false);
               }
            }
            %id.aimAt($gspPlayer);
      }
   }
   
   // Don't loop if global variable stops A.I.
   if($gspStopAI == false) { 
      // Dont loop if single enemy is dead  
      if(%id.getState() $= "Dead") {
         %id.isDead = true;
         $Game::killCount++;
         echo("Kill counter is " @ $Game::killCount);
         //messageClient($gspPlayer,'',"KILL COUNTER IS " @ $Game::killCount);
      }
   }
   
   // A.I. loop
   if($gspStopAI == false) {
      if(%id.isDead == false) {
         schedule(400, 0, updateBot, %id);
      }
   }   
}

function AIPlayer::spawnOnPath(%name,%node,%style)
{
   // Spawn a player and place him on the first node of the path
   if (!isObject(%node))
      return 0;
   //%node = %path.getObject(0);
   %player = AIPlayer::spawn(%name,%node.getTransform(),%style);
   return %player;
}


//-----------------------------------------------------------------------------
// AIPlayer methods 
//-----------------------------------------------------------------------------

function AIPlayer::followPath(%this,%path,%node)
{
   // Start the player following a path
   %this.stopThread(0);
   if (!isObject(%path)) {
      %this.path = "";
      return;
   }
   if (%node > %path.getCount() - 1)
      %this.targetNode = %path.getCount() - 1;
   else
      %this.targetNode = %node;
   if (%this.path $= %path)
      %this.moveToNode(%this.currentNode);
   else {
      %this.path = %path;
      %this.moveToNode(0);
   }
}

function AIPlayer::moveToNextNode(%this)
{
   if (%this.targetNode < 0 || %this.currentNode < %this.targetNode) {
      if (%this.currentNode < %this.path.getCount() - 1)
         %this.moveToNode(%this.currentNode + 1);
      else
         %this.moveToNode(0);
   }
   else
      if (%this.currentNode == 0)
         %this.moveToNode(%this.path.getCount() - 1);
      else
         %this.moveToNode(%this.currentNode - 1);
}

function AIPlayer::moveToNode(%this,%index)
{
   // Move to the given path node index
   %this.currentNode = %index;
   %node = %this.path.getObject(%index);
   %this.setMoveDestination(%node.getTransform(), %index == %this.targetNode);
}


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

function AIPlayer::pushTask(%this,%method)
{
   if (%this.taskIndex $= "") {
      %this.taskIndex = 0;
      %this.taskCurrent = -1;
   }
   %this.task[%this.taskIndex] = %method; 
   %this.taskIndex++;
   if (%this.taskCurrent == -1)
      %this.executeTask(%this.taskIndex - 1);
}

function AIPlayer::clearTasks(%this)
{
   %this.taskIndex = 0;
   %this.taskCurrent = -1;
}

function AIPlayer::nextTask(%this)
{
   if (%this.taskCurrent != -1)
      if (%this.taskCurrent < %this.taskIndex - 1)
         %this.executeTask(%this.taskCurrent++);
      else
         %this.taskCurrent = -1;
}

function AIPlayer::executeTask(%this,%index)
{
   %this.taskCurrent = %index;
   eval(%this.getId() @ "." @ %this.task[%index] @ ";");
}


//-----------------------------------------------------------------------------

function AIPlayer::singleShot(%this)
{
   // The shooting delay is used to pulse the trigger
   %this.setImageTrigger(0,true);
   %this.setImageTrigger(0,false);
   //%this.trigger = %this.schedule(%this.shootingDelay,singleShot);
}


//-----------------------------------------------------------------------------

function AIPlayer::wait(%this,%time)
{
   %this.schedule(%time * 1000,"nextTask");
}

function AIPlayer::done(%this,%time)
{
   %this.schedule(0,"delete");
}

function AIPlayer::fire(%this,%bool)
{
   if (%bool) {
      cancel(%this.trigger);
      %this.singleShot();
   }
   else
      cancel(%this.trigger);
   %this.nextTask();
}

function AIPlayer::aimAt(%this,%object)
{
   //echo("Aim: " @ %object);
   %random1 = getRandom(-%this.accuracy, %this.accuracy);
   %random2 = getRandom(-%this.accuracy, %this.accuracy);
   %vector = %random1 @ " " @ %random2 @ " 2.0";
   %this.setAimObject(%object,%vector);
   //echo(%vector);
   %this.nextTask();
}

function AIPlayer::animate(%this,%seq)
{
   //%this.stopThread(0);
   //%this.playThread(0,%seq);
   %this.setActionThread(%seq);
}


//-----------------------------------------------------------------------------

function AIManager::think(%this)
{
   // We could hook into the player's onDestroyed state instead of
   // having to "think", but thinking allows us to consider other
   // things...
   //if (!isObject(%this.player))
      //%this.player = %this.spawn();
   //%this.schedule(500,think);
   %this.spawn();
}

function AIManager::spawn(%this)
{
   if ($Server::MissionFile $= "starter.fps/data/missions/culdesac.mis")
		{
			//CULDESAC-----------------------------
			// Initialize enemies and correct path
			%path = "MissionGroup/Paths/Path1";
			%node1 = %path.getObject(0);
			%node2 = %path.getObject(1);
			%node3 = %path.getObject(2);
			%node4 = %path.getObject(3);
			%node5 = %path.getObject(4);
			%node6 = %path.getObject(5);
			%player1 = AIPlayer::spawnOnPath("Lacky",%node1,0);
			%player2 = AIPlayer::spawnOnPath("Lacky",%node2,0);
			%player3 = AIPlayer::spawnOnPath("Lacky",%node3,0);
			%player4 = AIPlayer::spawnOnPath("Butch",%node4,1);
			%player5 = AIPlayer::spawnOnPath("Lacky",%node5,0);
			%player6 = AIPlayer::spawnOnPath("Timmy",%node6,2);
		}
   else if ($Server::MissionFile $= "starter.fps/data/missions/mcgreary.mis")	
        {
			//MCGREARY----------------------------
			// Initialize enemies and correct path
			%path = "MissionGroup/Paths/Path1";
			%node1 = %path.getObject(0);
			%node2 = %path.getObject(1);
			%node3 = %path.getObject(2);
			%node4 = %path.getObject(3);
			%node5 = %path.getObject(4);
			%node6 = %path.getObject(5);
			%node25 = %path.getObject(24);
			%node26 = %path.getObject(25);
			%node27 = %path.getObject(26);
			%node28 = %path.getObject(27);
			%node29 = %path.getObject(28);
			%player1 = AIPlayer::spawnOnPath("Bully",%node1,1);
			%player2 = AIPlayer::spawnOnPath("Bully",%node2,1);
			%player3 = AIPlayer::spawnOnPath("Bully",%node3,1);
			%player4 = AIPlayer::spawnOnPath("Bully",%node4,1);
			%player5 = AIPlayer::spawnOnPath("Bully",%node5,1);
			%player6 = AIPlayer::spawnOnPath("Bully",%node6,1);
			%player25 = AIPlayer::spawnOnPath("Bully",%node25,1);
			%player26 = AIPlayer::spawnOnPath("Bully",%node26,1);
			%player27 = AIPlayer::spawnOnPath("Bully",%node27,1);
			%player28 = AIPlayer::spawnOnPath("Bully",%node28,1);
			%player29 = AIPlayer::spawnOnPath("Bully",%node29,1);
			
			%node7 = %path.getObject(6);
			%node8 = %path.getObject(7);
			%node9 = %path.getObject(8);
			%node10 = %path.getObject(9);
			%node11 = %path.getObject(10);
			%node12 = %path.getObject(11);
			%node13 = %path.getObject(12);
			//%node14 = %path.getObject(13);
			%node15 = %path.getObject(14);
			//%node16 = %path.getObject(15);
			%node17 = %path.getObject(16);
			%node18 = %path.getObject(17);
			//%node19 = %path.getObject(18);
			%node20 = %path.getObject(19);
			%node21 = %path.getObject(20);
			%node22 = %path.getObject(21);
			%node23 = %path.getObject(22);
			%node24 = %path.getObject(23);
			%player7 = AIPlayer::spawnOnPath("Lacky",%node7,0);
			%player8 = AIPlayer::spawnOnPath("Lacky",%node8,0);
			%player9 = AIPlayer::spawnOnPath("Lacky",%node9,0);
			%player10 = AIPlayer::spawnOnPath("Lacky",%node10,0);
			%player11 = AIPlayer::spawnOnPath("Lacky",%node11,0);
			%player12 = AIPlayer::spawnOnPath("Lacky",%node12,0);
			%player13 = AIPlayer::spawnOnPath("Lacky",%node13,0);
			//%player14 = AIPlayer::spawnOnPath("Lacky",%node14,0);
			%player15 = AIPlayer::spawnOnPath("Lacky",%node15,0);
			//%player16 = AIPlayer::spawnOnPath("Lacky",%node16,0);
			%player17 = AIPlayer::spawnOnPath("Lacky",%node17,0);
			%player18 = AIPlayer::spawnOnPath("Lacky",%node18,0);
			//%player19 = AIPlayer::spawnOnPath("Lacky",%node19,0);
			%player20 = AIPlayer::spawnOnPath("Lacky",%node20,0);
			%player21 = AIPlayer::spawnOnPath("Lacky",%node21,0);
			%player22 = AIPlayer::spawnOnPath("Lacky",%node22,0);
			%player23 = AIPlayer::spawnOnPath("Lacky",%node23,0);
			%player24 = AIPlayer::spawnOnPath("Lacky",%node24,0);

			%node30 = %path.getObject(29);
			%node31 = %path.getObject(30);
			%player30 = AIPlayer::spawnOnPath("McGreary",%node30,2);
			%player31 = AIPlayer::spawnOnPath("Chuck",%node31,2);
		}   
   else if ($Server::MissionFile $= "starter.fps/data/missions/pond.mis")
		{
			//POND------------------------------------
			//Paste this under
			// Initialize enemies and correct path
			%path = "MissionGroup/Paths/Path1";
			%node1 = %path.getObject(0);
			%node2 = %path.getObject(1);
			%node3 = %path.getObject(2);
			%node4 = %path.getObject(3);
			%node5 = %path.getObject(4);
			%node6 = %path.getObject(5);
			%player1 = AIPlayer::spawnOnPath("Bully",%node1,1);
			%player2 = AIPlayer::spawnOnPath("Bully",%node2,1);
			%player3 = AIPlayer::spawnOnPath("Bully",%node3,1);
			%player4 = AIPlayer::spawnOnPath("Bully",%node4,1);
			%player5 = AIPlayer::spawnOnPath("Bully",%node5,1);
			%player6 = AIPlayer::spawnOnPath("Whizzer",%node6,2);
		}
   else if ($Server::MissionFile $= "starter.fps/data/missions/school.mis")
		{
			// Initialize enemies and correct path
			%path = "MissionGroup/Paths/Path1";
			%node1 = %path.getObject(0);
			%node2 = %path.getObject(1);
			%node3 = %path.getObject(2);
			%node4 = %path.getObject(3);
			%node5 = %path.getObject(4);
			%node6 = %path.getObject(5);
			%node7 = %path.getObject(6);
			%node8 = %path.getObject(7);
			%friendlySchool = AIPlayer::spawnOnPath("Mr. Johnson",%node1,2);
			%sniperHill1 = AIPlayer::spawnOnPath("Bully",%node2,1);
			%gruntHill1 = AIPlayer::spawnOnPath("Lacky",%node3,0);
			%sniperHill2 = AIPlayer::spawnOnPath("Bully",%node4,1);
			%gruntHill2 = AIPlayer::spawnOnPath("Lacky",%node5,0);
			%gruntForest = AIPlayer::spawnOnPath("Lacky",%node6,0);
			%gruntGate = AIPlayer::spawnOnPath("Lacky",%node7,0);
			%gruntOpen = AIPlayer::spawnOnPath("Lacky",%node8,0);
		}
} 

   
   /*
   if (isObject(%player))
   {
      //%player.followPath("MissionGroup/Paths/Path1",-1);

      //%player.mountImage(CrossbowImage,0);
      //%player.setInventory(CrossbowAmmo,1000);
      return %player;
   }
   else
      return 0;
   */







