//-----------------------------------------------------------------------------
//SSD: Backhoe.cs
//SSD: Chuangang xie, Mike Manard expanded car.cs for backhoe
//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

// Information extacted from the shape.
//
// Wheel Sequences
//    spring#        Wheel spring motion: time 0 = wheel fully extended,
//                   the hub must be displaced, but not directly animated
//                   as it will be rotated in code.
// Other Sequences
//    steering       Wheel steering: time 0 = full right, 0.5 = center
//    breakLight     Break light, time 0 = off, 1 = breaking
//
// Wheel Nodes
//    hub#           Wheel hub, the hub must be in it's upper position
//                   from which the springs are mounted.
//
// The steering and animation sequences are optional.
// The center of the shape acts as the center of mass for the car.

//-----------------------------------------------------------------------------
datablock AudioProfile(BackHoeEngineSound)
{
   filename    = "~/data/sound/BBC/bicycle.ogg";
   description = AudiodefaultLooping3d;
   preload = true;
};

datablock ParticleData(BackHoeTireParticle)
{
   textureName          = "~/data/shapes/buggy/dustParticle";
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.1;
   inheritedVelFactor   = 0.1;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 400;
   colors[0]     = "0.46 0.36 0.26 1.0";
   colors[1]     = "0.46 0.46 0.36 0.0";
   sizes[0]      = 1.0;
   sizes[1]      = 4.0;
};

datablock ParticleEmitterData(BackHoeTireEmitter)
{
   ejectionPeriodMS = 20;
   periodVarianceMS = 10;
   ejectionVelocity = 1;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "BackHoeTireParticle";
};


//----------------------------------------------------------------------------

datablock WheeledVehicleTire(BackHoeFrontTire)//SSD: using smaller fronttire
{
   // Tires act as springs and generate lateral and longitudinal
   // forces to move the vehicle. These distortion/spring forces
   // are what convert wheel angular velocity into forces that
   // act on the rigid body.
   

   shapeFile= "~/data/shapes/Black_Ryane/br_backhoe_nowheel/fronttire1.dts"; 
   staticFriction = 1.2;
   kineticFriction = 1.15;

   // Spring that generates lateral tire forces
   lateralForce = 18000;
   lateralDamping = 6000;
   lateralRelaxation = 1;

   // Spring that generates longitudinal tire forces
   longitudinalForce = 18000;
   longitudinalDamping = 4000;
   longitudinalRelaxation = 1;
};

datablock WheeledVehicleTire(BackHoeRearTire)//SSD: bigger rear tire
{
   // Tires act as springs and generate lateral and longitudinal
   // forces to move the vehicle. These distortion/spring forces
   // are what convert wheel angular velocity into forces that
   // act on the rigid body.
   

   shapeFile= "~/data/shapes/Black_Ryane/br_backhoe_nowheel/reartire1.dts";
   staticFriction = 4.2;
   kineticFriction = 4.15;

   // Spring that generates lateral tire forces
   lateralForce = 18000;
   lateralDamping = 6000;
   lateralRelaxation = 1;

   // Spring that generates longitudinal tire forces
   longitudinalForce = 18000;
   longitudinalDamping = 4000;
   longitudinalRelaxation = 1;

};

datablock WheeledVehicleSpring(BackHoeSpring) //SSD:
{
   // Wheel suspension properties
   length = 0.15;             // Suspension travel
   force = 2800;              // Spring force
   damping = 3600;             // Spring damping
   antiSwayForce = 3;         // Lateral anti-sway force
};

datablock WheeledVehicleData(BackHoe : BaseVehicle)//SSD: back hoe body
{
   shapeFile = "~/data/shapes/Black_Ryane/br_backhoe_nowheel/backhoe2.dts";

   maxDamage = 1.0;
   destroyedLevel = 0.5;

   maxSteeringAngle = 0.385;  // Maximum steering angle, should match animation
   tireEmitter = BackHoeTireEmitter; // All the tires use the same dust emitter

   // 3rd person camera settings
   cameraRoll = true;         // Roll the camera with the vehicle
   cameraMaxDist = 4.8;         // Far distance from vehicle
   cameraOffset = 1.5;        // Vertical offset from camera mount point
   cameraLag = 0.26;           // Velocity lag of camera
   cameraDecay = 1.25;        // Decay per sec. rate of velocity lag

   // Rigid Body
   mass = 380;
   massCenter = "0 0.9 0";    // Center of mass for rigid body
   massBox = "0 0 0";         // Size of box used for moment of inertia,
                              // if zero it defaults to object bounding box
   drag = 0.6;                // Drag coefficient
   bodyFriction = 0.6;
   bodyRestitution = 0.4;
   minImpactSpeed = 5;        // Impacts over this invoke the script callback
   softImpactSpeed = 5;       // Play SoftImpact Sound
   hardImpactSpeed = 15;      // Play HardImpact Sound
   integration = 8;           // Physics integration: TickSec/Rate
   collisionTol = 0.1;        // Collision distance tolerance
   contactTol = 0.1;          // Contact velocity tolerance

   // Engine
   engineTorque = 3300;       // Engine power
   engineBrake = 600;         // Braking when throttle is 0
   brakeTorque = 8000;        // When brakes are applied
   maxWheelSpeed = 50;        // Engine scale by current speed / max speed

   // Energy
   maxEnergy = 100;
   jetForce = 3000;
   minJetEnergy = 30;
   jetEnergyDrain = 2;

   // Sounds
//   jetSound = ScoutThrustSound;
   engineSound = backhoeEngineSound;
//   squealSound = ScoutSquealSound;
   softImpactSound=BaseVehicleSoftImpactSound;
   hardImpactSound=BaseVehicleHardImpactSound;
   wheelImpactSound=BaseVehicleWheelImpactSound;

//   explosion = VehicleExplosion;
};
//-----------------------------------------
function BackHoe::onAdd(%this,%obj) //SSD:specify adding wheels and spring for backhoe only 
{

   %obj.setWheelTire(0,BackHoeFrontTire);
   %obj.setWheelTire(1,BackHoeFrontTire);
   %obj.setWheelTire(2,BackHoeRearTire);
   %obj.setWheelTire(3,BackHoeRearTire);

   // Setup the car with some defaults tires & springs
   for (%i = %obj.getWheelCount() - 1; %i >= 0; %i--) {
      %obj.setWheelSpring(%i,BackHoeSpring);
      %obj.setWheelPowered(%i,false);
   }

   // Steer front tires
   %obj.setWheelSteering(0,1);
   %obj.setWheelSteering(1,1);

   // power all 4 wheels...
   %obj.setWheelPowered(0,true);
   %obj.setWheelPowered(1,true);
   %obj.setWheelPowered(2,true);
   %obj.setWheelPowered(3,true);
}

// SSD: catch spawnOnPath call and sent it to parent (BaseVehicle)
function BackHoe::spawnOnPath(%name,%path)
{BaseVehicle::spawnOnPath(%name,%path);}

// SSD: catch events and send them to parent (BaseVehicle)
function BackHoe::onReachDestination(%this,%obj)
{BaseVehicle::onReachDestination(%this,%obj);}

function BackHoe::onEndOfPath(%this,%obj,%path)
{BaseVehicle::onEndOfPath(%this,%obj,%path);}

function BackHoe::onEndSequence(%this,%obj,%slot)
{BaseVehicle::onEndSequence(%this,%obj,%slot);}